//
// C++ Implementation: %{MODULE}
//
// Description:
//
//
// Author: %{AUTHOR} <%{EMAIL}>, (C) %{YEAR}
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef WW_NO_TASKPANEL

#include "qwwtaskpaneliface.h"
#include "qwwtaskpanel.h"
#include <QtDesigner/QExtensionFactory>
#include <QtDesigner/QExtensionManager>
#include <QtDesigner/QDesignerFormEditorInterface>
#include <QtDesigner/QDesignerFormWindowInterface>
#include <QtDesigner/QDesignerContainerExtension>
#include <QtDesigner/QDesignerPropertySheetExtension>

#include "qwwtaskpanel/qwwtaskpanel_p.h"
#include <QChildEvent>
#include <QMainWindow>
#include <QMenuBar>

#include "wwwidgets.h"
// #include "propertymatcher.h"

QwwTaskPanelIface::QwwTaskPanelIface(QObject *parent)
        : QObject(parent), QDesignerCustomWidgetInterface() {
    m_initialized = false;
}


QwwTaskPanelIface::~QwwTaskPanelIface() {}




QString QwwTaskPanelIface::includeFile() const {
    return "qwwtaskpanel.h";
}

QWidget * QwwTaskPanelIface::createWidget(QWidget * parent) {
    QwwTaskPanel *panel = new QwwTaskPanel(parent);
    connect(panel, SIGNAL(currentIndexChanged(int)),
            this, SLOT(currentIndexChanged(int)));
    QWidget *w = qFindChild<QWidget*>(panel, "ww_taskpanel_panel");
    if (w) {
        w->installEventFilter(this);
    }

    return panel;
}

QIcon QwwTaskPanelIface::icon() const {
    return QPixmap(":/trolltech/formeditor/images/widgets/widgetstack.png");
}

void QwwTaskPanelIface::initialize(QDesignerFormEditorInterface * core) {
    if (m_initialized)
        return;
    formEditor = core;
    QExtensionManager *manager = formEditor->extensionManager();
    QExtensionFactory *factory = new QwwWidgetsExtensionFactory(manager);

    Q_ASSERT(manager != 0);
    manager->registerExtensions(factory, Q_TYPEID(QDesignerContainerExtension));
    manager->registerExtensions(factory, Q_TYPEID(QDesignerPropertySheetExtension));
    QTimer::singleShot(1000, this, SLOT(execute()));
    m_initialized = true;
}

QwwTaskPanelContainerExtension::QwwTaskPanelContainerExtension(QwwTaskPanel * widget, QObject * ) {
    myWidget = widget;
}

QWidget * QwwTaskPanelContainerExtension::widget(int index) const {
    return myWidget->task(index);
}

void QwwTaskPanelContainerExtension::setCurrentIndex(int index) {
    myWidget->setCurrentIndex(index);
}

void QwwTaskPanelContainerExtension::remove(int index) {
    myWidget->removeTask(index);
}

void QwwTaskPanelContainerExtension::insertWidget(int index, QWidget * widget) {
    myWidget->insertTask(index, widget);
}

int QwwTaskPanelContainerExtension::currentIndex() const {
    return myWidget->currentIndex();
}

int QwwTaskPanelContainerExtension::count() const {
    return myWidget->taskCount();
}

void QwwTaskPanelContainerExtension::addWidget(QWidget * widget) {
    myWidget->addTask(widget);
}

QString QwwTaskPanelIface::domXml() const {
    return QString("\
                   <widget class=\"QwwTaskPanel\" name=\"taskPanel\">\
                   <widget class=\"QWidget\" name=\"page\" />\
                   </widget>\
                   ");
}

void QwwTaskPanelIface::currentIndexChanged(int )
{
    QwwTaskPanel *widget = qobject_cast<QwwTaskPanel*>(sender());
    if (widget) {
        QDesignerFormWindowInterface *form;
        form = QDesignerFormWindowInterface::findFormWindow(widget);
        if (form)
            form->emitSelectionChanged();
    }
}

bool QwwTaskPanelIface::eventFilter(QObject * , QEvent * e)
{
    if(e->type() == QEvent::ChildAdded){
        QChildEvent *chev = static_cast<QChildEvent*>(e);
        Task *tsk = qobject_cast<Task*>(chev->child());
        if(tsk){
            tsk->body()->setMinimumHeight(200);
        }
    }
    return false;
}

void QwwTaskPanelIface::execute()
{
    if(formEditor->topLevel()){
        QMainWindow *mw = dynamic_cast<QMainWindow*>(formEditor->topLevel());
        if(!mw) return;
        wwWidgetsMenu *menu = new wwWidgetsMenu("wwWidgets");
        mw->menuBar()->addMenu(menu);
    }

}
const int tp_propcnt = 3;

int QwwTaskPanelSheetExtension::count() const
{

    return m_other ? m_other->count() + tp_propcnt : tp_propcnt;
}

bool QwwTaskPanelSheetExtension::isVisible(int index) const
{
    if(index>=myindex) return true;
    if(m_other){
        return m_other->isVisible(index);
    }
    return false;
}

bool QwwTaskPanelSheetExtension::isChanged(int index) const
{
    if(index>=myindex) return false;
    if(m_other){
        return m_other->isChanged(index);
    }
    return false;
}

bool QwwTaskPanelSheetExtension::isAttribute(int index) const
{
    if(index>=myindex) return true;
    if(m_other){
        return m_other->isAttribute(index);
    }
    return false;
}

int QwwTaskPanelSheetExtension::indexOf(const QString & name) const
{
    if(name=="currentTaskTitle") return myindex+2;
    if(name=="currentTaskName") return myindex;
    if(name=="currentTaskIcon") return myindex+1;
    if(m_other){
        return m_other->indexOf(name);
    }
    return -1;
}

bool QwwTaskPanelSheetExtension::hasReset(int index) const
{
    if(index==myindex+1) return true;
    if(index==myindex+2) return false;
    if(index==myindex) return true;
    if(m_other){
        return m_other->hasReset(index);
    }
    return false;
}

QString QwwTaskPanelSheetExtension::propertyName(int index) const
{
    if(index==myindex+2) return "currentTaskTitle";
    if(index==myindex+1) return "currentTaskIcon";
    if(index==myindex) return "currentTaskName";
    if(m_other){
        return m_other->propertyName(index);
    }
    return QString::null;
}

QString QwwTaskPanelSheetExtension::propertyGroup(int index) const
{
    if(index>=myindex && index<myindex+tp_propcnt) return "QwwTaskPanel";
    if(m_other){
        return m_other->propertyGroup(index);
    }
    return "";
}

QVariant QwwTaskPanelSheetExtension::property(int index) const
{
    if(index==myindex+2) return m_panel->currentTask()->windowTitle();
    if(index==myindex+1) return m_panel->currentTask()->windowIcon();
    if(index==myindex+0) return m_panel->currentTask()->objectName();
    if(m_other){
        return m_other->property(index);
    }
    return QVariant();
}

void QwwTaskPanelSheetExtension::setAttribute(int index, bool attribute)
{
    if(index==myindex) return;
    if(index==myindex+1) return;
    if(index==myindex+2) return;
    if(m_other){
        return m_other->setAttribute(index, attribute);
    }
}

bool QwwTaskPanelSheetExtension::reset(int index)
{
    if(index==myindex+1){
        QWidget *task = m_panel->currentTask();
        m_panel->setTaskIcon(m_panel->currentIndex(), QIcon());
        setPropertyChanged(task, "windowIcon");
        return true;

    }
    if(index==myindex){
        QWidget *task = m_panel->currentTask();
        QDesignerFormWindowInterface *form = QDesignerFormWindowInterface::findFormWindow(m_panel);
    if (form && task) {
        QDesignerFormEditorInterface *editor = form->core();
        QExtensionManager *manager = editor->extensionManager();
        QDesignerPropertySheetExtension *sheet;
        sheet = qt_extension<QDesignerPropertySheetExtension*>(manager, task);
        int propertyIndex = sheet->indexOf("objectName");
        if(sheet->hasReset(propertyIndex))
        return sheet->reset(propertyIndex);
        else return false;
    }
    }
    if(index==myindex+2) return false;
    if(m_other){
        return m_other->reset(index);
    }
    return false;
}

void QwwTaskPanelSheetExtension::setProperty(int index, const QVariant & value)
{
    if(index==myindex){
        QWidget *task = m_panel->currentTask();
        m_panel->setTaskName(m_panel->currentIndex(), value.toString());
        setPropertyChanged(task, "objectName");
        return;
    }
    if(index==myindex+1){
        QWidget *task = m_panel->currentTask();
        m_panel->setTaskIcon(m_panel->currentIndex(), qvariant_cast<QIcon>(value));
//         task->setWindowTitle
        setPropertyChanged(task, "windowIcon");
        return;
    }
    if(index==myindex+2){
        QWidget *task = m_panel->currentTask();
        m_panel->setTaskTitle(m_panel->currentIndex(), value.toString());
        setPropertyChanged(task, "windowTitle");
        return;
    }
    if(m_other){
        return m_other->setProperty(index, value);
    }
}

void QwwTaskPanelSheetExtension::setChanged(int index, bool changed)
{
    if(index==myindex) return;
    if(index==myindex+1) return;
    if(index==myindex+2) return;
    if(m_other){
        return m_other->setChanged(index, changed);
    }
}

void QwwTaskPanelSheetExtension::setPropertyGroup(int index, const QString & group)
{
    if(index==myindex) return;
    if(index==myindex+2) return;
    if(index==myindex+1) return;
    if(m_other){
        return m_other->setPropertyGroup(index, group);
    }
}

void QwwTaskPanelSheetExtension::setVisible(int index, bool visible)
{
    if(index==myindex) return;
    if(index==myindex+1) return;
    if(index==myindex+2) return;
    if(m_other){
        return m_other->setVisible(index, visible);
    }
}

// class TaskPanelPropertyMatcher : public PropertyMatcher {
// public:
//     TaskPanelPropertyMatcher(QwwTaskPanel *panel){
//         m_panel = panel;
//     }
//     void setProperty(QObject *o, const QString &name, const QVariant &value){
//
//         QWidget *task = m_panel->currentTask();
//         int index = m_panel->currentIndex();
//         QDesignerFormWindowInterface *form = QDesignerFormWindowInterface::findFormWindow(m_panel);
//         if (form && task) {
//             QDesignerFormEditorInterface *editor = form->core();
//             QExtensionManager *manager = editor->extensionManager();
//             QDesignerPropertySheetExtension *sheet;
//             sheet = qt_extension<QDesignerPropertySheetExtension*>(manager, task);
//             int propertyIndex = sheet->indexOf(QLatin1String("windowIcon"));
//             sheet->setChanged(propertyIndex, true);
//         }
//
//     }
// };


QwwTaskPanelSheetExtension::QwwTaskPanelSheetExtension(QWidget *w, QObject *other, QObject *parent) : QObject(parent)
{

    m_other = qobject_cast<QDesignerPropertySheetExtension*>(other);
    myindex = m_other->count();
    m_panel = qobject_cast<QwwTaskPanel*>(w);
    //     m_matcher = new TaskPanelPropertyMatcher(m_panel);
}

QwwTaskPanelSheetExtension::~ QwwTaskPanelSheetExtension()
{
    //     delete m_matcher;
}

void QwwTaskPanelSheetExtension::setPropertyChanged(QWidget * task, const QString & propName, const QVariant &value)
{
    QDesignerFormWindowInterface *form = QDesignerFormWindowInterface::findFormWindow(m_panel);
    if (form && task) {
        QDesignerFormEditorInterface *editor = form->core();
        QExtensionManager *manager = editor->extensionManager();
        QDesignerPropertySheetExtension *sheet;
        sheet = qt_extension<QDesignerPropertySheetExtension*>(manager, task);
        int propertyIndex = sheet->indexOf(propName);
        if(value.isValid())
            sheet->setProperty(propertyIndex, value);
        sheet->setChanged(propertyIndex, true);
    }

}

#endif
