//
// C++ Implementation: %{MODULE}
//
// Description:
//
//
// Author: %{AUTHOR} <%{EMAIL}>, (C) %{YEAR}
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef WW_NO_COLORBUTTON
#include "qwwcolorbutton.h"
#include <QStandardItemModel>
#include <QAbstractItemDelegate>
#include <QPainter>
#include <QListView>
#include <QLayout>
#include <cmath>
#include <QScrollBar>
#include <QColorDialog>
#include "wwglobal_p.h"
#include <QtDebug>
#include <QEvent>



/**
 * \internal
 * @clas ColorModel
 */
class ColorModel : public QStandardItemModel {
public:
    ColorModel(QObject *parent=0) : QStandardItemModel(parent) {
        setColumnCount(1);
    }
};

/**
 * @internal
 * Delegate for the colour chooser
 */
class ColorDelegate : public QAbstractItemDelegate {
public:
    ColorDelegate(QObject *parent = 0) : QAbstractItemDelegate(parent) {}
    void paint ( QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index ) const {
        if (!index.isValid()) return;
        QColor c = qvariant_cast<QColor>(index.data(Qt::DecorationRole));
//         QBrush b = painter->brush();
        painter->save();
        painter->setBrush(c);
        painter->drawRect(option.rect);
        if(option.state & QStyle::State_Selected){
            QPen p = painter->pen();
            p.setWidth(2);
            painter->setPen(p);
            painter->drawRect(option.rect.adjusted(1,1,0,0));
        }

//         painter->setBrush(b);
painter->restore();

    }
    QSize sizeHint ( const QStyleOptionViewItem & /*option*/, const QModelIndex & /*index*/ ) const {
        return QSize(16,16);
    }
};

/**
 * \internal
 */
class QwwColorPopup : public QWidget {
public:
    QwwColorPopup(ColorModel *model, QWidget *parent = 0) : QWidget(parent, Qt::Popup) {
        QVBoxLayout *l = new QVBoxLayout(this);
        l->setSpacing(1);
        l->setMargin(0);
        view = new QListView;
        view->setFlow(QListView::LeftToRight);
        view->setResizeMode(QListView::Adjust);
        view->setWrapping(true);
        view->setUniformItemSizes(true);
        view->setMouseTracking(true);

//         view->viewport()->setMaximumWidth(60);
//view->setMaximumWidth(72);
        l->addWidget(view);
        button = new QPushButton;
        button->setText(tr("Other"));
        button->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
        l->addWidget(button);
        view->setItemDelegate(new ColorDelegate(this));
        view->setModel(model);
        if(parent)
        connect(button, SIGNAL(clicked()), parent, SLOT(_q_colorDialogRequested()));
        connect(view, SIGNAL(entered(const QModelIndex &)), parent, SLOT(_q_setCurrentIndex(const QModelIndex &)));
    }
    QListView *colorView() const {
        return view;
    }

private:
    QListView *view;
    QPushButton *button;
};

class QwwColorButtonPrivate : public QwwPrivate {
public:
    QwwColorButtonPrivate(QwwColorButton *q) : QwwPrivate(q) {
        popup = 0;
        model = 0;
    }
    QwwColorPopup *popup;
    ColorModel *model;
    QColor curCol;
    QString curNam;
};

/**
 *  @class  QwwColorButton
 *  @brief  Button acting as a colour chooser
 *
 *          This widget provides a push button that can be used
 *          to choose a colour from either a predefined set of colours
 *          or a custom colour using a QColorDialog.
 *
 *          Colours can be added to the button with addColor(), setColors()
 *          or setStandardColors().
 *
 *
 * @fn      QwwColorButton::colorPicked(QColor)
 * @brief   a colour has been chosen
 *
 *          This signal is emitted when a colour is picked and displayed
 *          on the button
 *
 */

QwwColorButton::QwwColorButton(QWidget *parent)
        : QPushButton(parent), QwwPrivatable(new QwwColorButtonPrivate(this)) {
    Q_D(QwwColorButton);
    d->model = new ColorModel(this);
    connect(this, SIGNAL(clicked()), SLOT(_q_clicked()));
    setStandardColors();
}


/**
 * @brief Destructor
 *
 * Destroys the button.
 */
QwwColorButton::~QwwColorButton() {}

/**
 * \internal
 */
void QwwColorButton::_q_clicked() {
    Q_D(QwwColorButton);
    if (!d->popup) {
        d->popup = new QwwColorPopup(d->model, this);
//          m_popup->layout()->setSizeConstraint(QLayout::SetFixedSize);
        connect(d->popup->colorView(), SIGNAL(clicked(const QModelIndex&)), SLOT(_q_activated(const QModelIndex&)));
    }
    QPoint p = rect().bottomLeft();
    p = mapToGlobal(p);
    d->popup->move(p);
//     m_popup->setFixedWidth(81/*+m_popup->colorView()->verticalScrollBar()->width()*/);
//     m_popup->setFixedHeight(80+28);
//     m_popup->colorView()->viewport()->resize(80, 80);
    d->popup->show();

}

/**
 * \internal
 *
 */
void QwwColorButton::_q_activated(const QModelIndex &ind) {
    Q_D(QwwColorButton);
    d->popup->hide();
    QColor c = qvariant_cast<QColor>(ind.data(Qt::DecorationRole));
    d->curCol = c;
    d->curNam = ind.data(Qt::ToolTipRole).toString();
    QPixmap px(12,12);
    QPainter pt(&px);
    pt.setBrush(c);
    pt.drawRect(0, 0, 11, 11);
    setIcon(px);
    setText(ind.data(Qt::ToolTipRole).toString());
    emit colorPicked(c);
}

/**
 * @brief Returns the currently chosen colour
 */
QColor QwwColorButton::currentColor() const {
    Q_D(const QwwColorButton);
    return d->curCol;
}

/**
 * @brief Adds a \a colour with a \a name to the list of colours to choose from
 * @param colour colour to add
 * @param name name to associate with the colour
 */
void QwwColorButton::addColor(const QColor & c, const QString & n) {
    Q_D(QwwColorButton);
    QStandardItem *item = new QStandardItem;
    item->setData(c, Qt::DecorationRole);
    item->setData(n, Qt::ToolTipRole);
    d->model->appendRow(item);
}

/**
 * @brief Clears the colour list
 */
void QwwColorButton::clear() {
    Q_D(QwwColorButton);
    d->model->clear();
}

/**
 * @brief Sets standard Qt colours as the list of colours to choose from
 */
void QwwColorButton::setStandardColors() {
    QStringList clist = QColor::colorNames();
    clear();
    foreach(QString col, clist) {
        QColor c;
        c.setNamedColor(col);
        addColor(c, col);
    }
}

/**
 * @brief list of colours known to the button.
 *
 */
QList< QColor > QwwColorButton::colors() const {
    return QList<QColor>();

}

/**
 *
 * @param list
 */
void QwwColorButton::setColors(const QList< QColor > &list) {
    clear();
    foreach(QColor col, list) {
        addColor(col, QString());
    }
}

/**
 * \internal
 */
void QwwColorButton::_q_colorDialogRequested()
{
    Q_D(QwwColorButton);
    QColor c = QColorDialog::getColor(Qt::white, this);
    if(c.isValid()){
        addColor(c, tr("Custom color"));
        _q_activated(d->model->index(d->model->rowCount()-1, 0));
    }
}

/**
 * Sets the current colour
 * @param c
 */
void QwwColorButton::setCurrentColor(const QColor & c)
{
    Q_D(QwwColorButton);
    if(c==d->curCol) return;
    //TODO Find colour in model
    d->curCol = c;
    emit colorPicked(c);
}

void QwwColorButton::_q_setCurrentIndex(const QModelIndex &index)
{
    Q_D(QwwColorButton);
    d->popup->colorView()->selectionModel()->select(index, QItemSelectionModel::ClearAndSelect);
}


#endif
