//
// C++ Implementation: %{MODULE}
//
// Description:
//
//
// Author: %{AUTHOR} <%{EMAIL}>, (C) %{YEAR}
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifndef WW_NO_HUESATPICKER

#include "qwwhuesatpicker.h"
#include <QPainter>
#include <QImage>
#include <QMouseEvent>
#include <QResizeEvent>
#include <QStyleOptionFrame>
#include "wwglobal_p.h"

const int wid = 360;
const int hei = 256;


class QwwHueSatPickerPrivate : public QwwPrivate {
public:
    QwwHueSatPickerPrivate(QwwHueSatPicker *q) : QwwPrivate(q) {
        m_minS = 0;
        m_maxS = 255;
        m_minH = 0;
        m_maxH = 359;
        m_v = 200;
        m_pt = QPoint(0, 0);
    }
    int hueFromX(int) const;
    int satFromY(int) const;
    int satToY(int) const;
    int hueToX(int) const;
    void buildPixmap();
    int m_minS;
    int m_maxS;
    int m_minH;
    int m_maxH;
    int m_v;
    QPoint m_pt;
    QPixmap px;
    QSize pxs;
    QColor m_color;
    WW_DECLARE_PUBLIC(QwwHueSatPicker);
};



/**
 *  @class QwwHueSatPicker
 *  @brief A widget allowing color picking in a form of a hue-saturation rectangle
 *
 *
 */


/**
 *
 * @param parent
 */
QwwHueSatPicker::QwwHueSatPicker(QWidget *parent)
        : QFrame(parent), QwwPrivatable(new QwwHueSatPickerPrivate(this)) {
    Q_D(QwwHueSatPicker);
    d->buildPixmap();
    setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
}


/**
 * @brief Destructor
 */
QwwHueSatPicker::~QwwHueSatPicker() {}

/**
 *
 * @param v
 */
void QwwHueSatPicker::setValue(int v) {
    Q_D(QwwHueSatPicker);
    if (d->m_v==v) return;
    if (v<0 || v>255) return;
    d->m_v = v;
    emit valueChanged(v);
    d->pxs=QSize();
    d->buildPixmap();
    update();

}

/**
 * \internal
 * @return
 */
QSize QwwHueSatPicker::sizeHint() const {
    Q_D(const QwwHueSatPicker);
    return QSize((d->m_maxH-d->m_minH)+2*frameWidth(), (d->m_maxS-d->m_minS)+2*frameWidth());
}

/**
 * \internal
 * @return
 */
QSize QwwHueSatPicker::minimumSizeHint() const {
    Q_D(const QwwHueSatPicker);
    return QSize(qMin((d->m_maxH-d->m_minH)/4, 40)+2*frameWidth(), qMin((d->m_maxS-d->m_minS)/4, 40)+2*frameWidth());
}

/**
 * \internal
 */
void QwwHueSatPickerPrivate::buildPixmap() {
    Q_Q(QwwHueSatPicker);
    int cy = q->contentsRect().height();
    int cx = q->contentsRect().width();
    QImage img(cx, cy, QImage::Format_RGB32);
    for (int y=0; y<cy;y++)
        for (int x=0; x<cx;x++) {
            QColor c;
            c.setHsv(hueFromX(x), satFromY(y), m_v);
            img.setPixel(x, y, c.rgb());
        }
    px = QPixmap::fromImage(img);
    pxs = px.size();
}

/**
 * \internal
 */
void QwwHueSatPicker::paintEvent(QPaintEvent *) {
    Q_D(QwwHueSatPicker);
    QPainter p(this);
    drawFrame(&p);
    QRect rct = contentsRect();
    QStyleOptionFrame opt;
    opt.initFrom(this);
    if (opt.state & QStyle::State_Enabled) {
        p.drawPixmap(rct.topLeft(), d->px);
        drawCrosshair(&p, d->m_pt);
    } else {
        QIcon i(d->px);
        i.paint(&p, rct, 0, QIcon::Disabled);
    }
}

/**
 * \internal
 */
int QwwHueSatPickerPrivate::hueFromX(int x) const {
    Q_Q(const QwwHueSatPicker);
    return m_maxH-x*(m_maxH-m_minH)/q->contentsRect().width();
}

/**
 * \internal
 */
int QwwHueSatPickerPrivate::satFromY(int y) const {
    \
    Q_Q(const QwwHueSatPicker);
    return m_maxS-y*(m_maxS-m_minS)/q->contentsRect().height();
}

/**
 * \internal
 * @param s
 * @return
 */
int QwwHueSatPickerPrivate::satToY(int s) const {
    Q_Q(const QwwHueSatPicker);
    float ys = q->contentsRect().height()*1.0/(m_maxS-m_minS);
    float dist = s*ys;
    return (int)(q->contentsRect().height() - dist);
}


/**
 * \internal
 * @param h
 * @return
 */
int QwwHueSatPickerPrivate::hueToX(int h) const {
    Q_Q(const QwwHueSatPicker);
    float xs = q->contentsRect().width()*1.0/(m_maxH-m_minH); // pixels per hue
    float dist = h*xs;
    return (int)(q->contentsRect().width()-dist);
}


/**
 * \internal
 */
void QwwHueSatPicker::mousePressEvent(QMouseEvent * me) {
    Q_D(QwwHueSatPicker);
    QPoint pt = me->pos();
    int h = d->hueFromX(pt.x()-contentsRect().x());
    int s = d->satFromY(pt.y()-contentsRect().y());
    if (h<0 || s<0 || h>359 || s>255) return;
    emit colorPicked(QColor::fromHsv(h, s, d->m_v));
    d->m_pt = pt;
    update();
}

/**
 * \internal
 */
void QwwHueSatPicker::mouseMoveEvent(QMouseEvent * me) {
    Q_D(QwwHueSatPicker);
    QPoint pt = me->pos();
    if (!contentsRect().contains(pt)) return;
    int h = d->hueFromX(pt.x()-contentsRect().x());
    int s = d->satFromY(pt.y()-contentsRect().y());
    if (h<0 || s<0 || h>359 || s>255) return;
    emit colorPicked(QColor::fromHsv(h, s, d->m_v));
    d->m_pt = pt;
    update();
}

/**
 *
 * @param p
 * @param pt
 */
void QwwHueSatPicker::drawCrosshair(QPainter * p, const QPoint & pt) {
    p->save();
    p->setPen(Qt::black);
    p->drawLine(pt-QPoint(0, -3), pt-QPoint(0, -1));
    p->drawLine(pt-QPoint(0, 1), pt-QPoint(0, 3));
    p->drawLine(pt-QPoint(-3, 0), pt-QPoint(-1, 0));
    p->drawLine(pt-QPoint(1, 0), pt-QPoint(3, 0));
    p->restore();
}

/**
 *
 * @param v
 */
void QwwHueSatPicker::setMinimumSat(int v) {
    Q_D(QwwHueSatPicker);
    setSatRange(v, qMax(d->m_maxS, v));
}

/**
 *
 * @param h
 */
void QwwHueSatPicker::setMinimumHue(int h) {
    Q_D(QwwHueSatPicker);
    setHueRange(h, qMax(d->m_maxH, h));
}

/**
 *
 * @param h
 */
void QwwHueSatPicker::setMaximumHue(int h) {
    Q_D(QwwHueSatPicker);
    setHueRange(qMin(d->m_minH, h), h);
}

/**
 *
 * @param mi
 * @param ma
 */
void QwwHueSatPicker::setHueRange(int mi, int ma) {
    Q_D(QwwHueSatPicker);
    if (mi<0 || ma < 0 || mi > 359 || ma>359 || mi >ma) return;
    bool maC = d->m_maxH!=ma;
    bool miC = d->m_minH!=mi;
    d->m_maxH = ma;
    d->m_minH = mi;
    if (maC) emit maximumHueChanged(ma);
    if (miC) emit minimumHueChanged(mi);
    if (miC || maC) emit hueRangeChanged(mi, ma);
    else return;

    updateGeometry();
    d->buildPixmap();
    update();
}

/**
 *
 * @param v
 */
void QwwHueSatPicker::setMaximumSat(int v) {
    Q_D(QwwHueSatPicker);
    setSatRange(qMin(d->m_minS, v), v);
}

/**
 *
 * @param mi
 * @param ma
 */
void QwwHueSatPicker::setSatRange(int mi, int ma) {
    Q_D(QwwHueSatPicker);
    if (mi<0 || ma < 0 || mi > 255 || ma>255 || mi >ma) return;
    bool maC = d->m_maxS!=ma;
    bool miC = d->m_minS!=mi;
    d->m_maxS = ma;
    d->m_minS = mi;
    if (maC) emit maximumSatChanged(ma);
    if (miC) emit minimumSatChanged(mi);
    if (miC || maC) emit satRangeChanged(mi, ma);
    else return;
//     m_maxS = ma;
//     m_minS = mi;
    updateGeometry();
    d->buildPixmap();
    update();
}

/**
 *
 * @param c
 */
void QwwHueSatPicker::setColor(const QColor & c) {
    Q_D(QwwHueSatPicker);
    int h, s, v;
    if (d->m_color==c) return;
    c.getHsv(&h, &s, &v);
    if (d->m_v!=v) setValue(v);
    int Hspan = d->m_maxH - d->m_minH;
    int Sspan = d->m_maxS - d->m_minS;
    if (s<(d->m_minS) || s>(d->m_maxS)) {
        int ss = qMax(0, s-Sspan/2);
        setSatRange(ss, ss+Sspan);
    }
    if (h<(d->m_minH) || h>(d->m_maxH)) {
        int hh = qMax(0, h-Hspan/2);
        setHueRange(hh, hh+Hspan);
    }
    d->m_pt = QPoint(d->hueToX(h), d->satToY(s));
    d->m_color = c;
    emit colorPicked(c);
    update();
}

/**
 * \internal
 */
void QwwHueSatPicker::resizeEvent(QResizeEvent *re) {
    Q_D(QwwHueSatPicker);
    QFrame::resizeEvent(re);
    d->buildPixmap();
    update();
}

/**
 *
 * @return
 */
QColor QwwHueSatPicker::color() const {
    Q_D(const QwwHueSatPicker);
    return d->m_color;
}

/**
 *
 * @return
 */
int QwwHueSatPicker::minimumSat() const {
    Q_D(const QwwHueSatPicker);
    return d->m_minS;
}

/**
 *
 * @return
 */
int QwwHueSatPicker::maximumSat() const {
    Q_D(const QwwHueSatPicker);
    return d->m_maxS;
}

/**
 *
 * @return
 */
int QwwHueSatPicker::maximumHue() const {
    Q_D(const QwwHueSatPicker);
    return d->m_maxH;
}

/**
 *
 * @return
 */
int QwwHueSatPicker::minimumHue() const {
    Q_D(const QwwHueSatPicker);
    return d->m_minH;
}

/**
 *
 * @return
 */
int QwwHueSatPicker::value() const {
    Q_D(const QwwHueSatPicker);
    return d->m_v;
}

#endif
