//
// C++ Implementation: %{MODULE}
//
// Description:
//
//
// Author: %{AUTHOR} <%{EMAIL}>, (C) %{YEAR}
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef WW_NO_HUESATRADIALPICKER

#include "qwwhuesatradialpicker.h"
#include <QPainter>
#include <QMouseEvent>
#include <cmath>
#include "wwglobal_p.h"

#ifndef M_PI
#define M_PI            3.14159265358979323846  /* pi */
#endif

class QwwHueSatRadialPickerPrivate : public QwwPrivate {
public:
    QwwHueSatRadialPickerPrivate(QwwHueSatRadialPicker *parent) : QwwPrivate(parent) {
        m_value = 255;
        for (int i=0;i<360;i++) {
            conical.setColorAt(i*1.0/360, QColor::fromHsv(i, 255, m_value));
        }
        conical.setColorAt(1, QColor::fromHsv(359, 255, m_value));
    }
    void buildPixmap();
    QPixmap px;
    QConicalGradient conical;
    int m_value;
    QPoint m_pt;
    QColor m_color;
    int radius(const QPoint &pt) const;
    int hue(const QPoint &pt) const;
    WW_DECLARE_PUBLIC(QwwHueSatRadialPicker);
};

/**
 *  @class QwwHueSatRadialPicker
 *  @brief A widget allowing color picking in a form of hue-saturation circle
 *
 *
 *
 */

/**
 *
 * @param parent
 */
QwwHueSatRadialPicker::QwwHueSatRadialPicker(QWidget *parent)
        : QWidget(parent), QwwPrivatable(new QwwHueSatRadialPickerPrivate(this)) {
    QSizePolicy policy(QSizePolicy::Fixed, QSizePolicy::Fixed);
    policy.setHeightForWidth(true);
    setSizePolicy(policy);
}


/**
 *
 */
QwwHueSatRadialPicker::~QwwHueSatRadialPicker() {}

void QwwHueSatRadialPicker::paintEvent(QPaintEvent *) {
    Q_D(QwwHueSatRadialPicker);
    if (d->px.isNull()) d->buildPixmap();
    QPainter p(this);
    p.drawPixmap(0, 0, d->px);
    drawCrosshair(&p, d->m_pt);
}

void QwwHueSatRadialPickerPrivate::buildPixmap() {
    // build huecircle
    Q_Q(QwwHueSatRadialPicker);
    conical.setCenter(q->rect().center());
    QImage huecircle(q->size(), QImage::Format_ARGB32);
    QPainter phcircle(&huecircle);
    phcircle.setRenderHint(QPainter::Antialiasing, true);
    phcircle.setBrush(QBrush(conical));
    phcircle.drawEllipse(q->rect().adjusted(1, 1, -1, -1));
    px = QPixmap::fromImage(huecircle);

    // alpha gradient
    QRadialGradient rg(q->rect().center(), q->width()/2-2, q->rect().center());

    for (float i=0;i<1;i+=0.1) {
        rg.setColorAt(i, QColor::fromHsv(0, 0, (int)(256*i)));
    }
    rg.setColorAt(1, Qt::white);

    // alpha channel
    QImage ac(q->size(), QImage::Format_RGB32);
    QPainter acpainter(&ac);
    acpainter.setPen(Qt::NoPen);
    acpainter.setBrush(QBrush(rg));
    acpainter.drawEllipse(q->rect().adjusted(1, 1, -1, -1));
    px.setAlphaChannel(QPixmap::fromImage(ac));

    // destination image
    QImage dst(q->size(), QImage::Format_ARGB32);
    //dst.fill(QColor(Qt::white).rgb());
    QPainter dstp(&dst);
    dstp.setBrush(QColor::fromHsv(0, 0, m_value));
    dstp.setRenderHint(QPainter::Antialiasing, true);
    dstp.drawEllipse(q->rect().adjusted(1, 1, -1, -1));
    dstp.setCompositionMode(QPainter::CompositionMode_SourceOver);
    dstp.drawPixmap(0, 0, px);
    /*dstp.setCompositionMode(QPainter::CompositionMode_Source);
    dstp.setBrush(Qt::NoBrush);
    dstp.setPen(Qt::black);
    dstp.drawEllipse(rect()); */
    px = QPixmap::fromImage(dst);
}

/**
 *
 * @param v
 */
void QwwHueSatRadialPicker::setValue(int v) {
    Q_D(QwwHueSatRadialPicker);
    if (v<0 || v>255 || v==d->m_value) return;
    d->m_value = v;
    emit valueChanged(v);
    d->conical.setStops(QGradientStops());
    for (int i=0;i<360;i++) {
        d->conical.setColorAt(i*1.0/360, QColor::fromHsv(i, 255, d->m_value));
    }
    d->conical.setColorAt(1, QColor::fromHsv(359, 255, d->m_value));
    d->buildPixmap();
    update();
}

/**
 *
 * @return
 */
int QwwHueSatRadialPicker::value() const {
    Q_D(const QwwHueSatRadialPicker);
    return d->m_value;
}

QSize QwwHueSatRadialPicker::sizeHint() const {
    return QSize(202, 202);
}

QSize QwwHueSatRadialPicker::minimumSizeHint() const {
    return QSize(202, 202);
}

int QwwHueSatRadialPicker::heightForWidth(int w) const {
    return w;
}

void QwwHueSatRadialPicker::resizeEvent(QResizeEvent *) {
    Q_D(QwwHueSatRadialPicker);
    if (width()!=height()) {
        int s = qMin(width(), height());
        resize(s,s);
        return;
    }
    d->buildPixmap();
    update();
}



void QwwHueSatRadialPicker::mousePressEvent(QMouseEvent * me) {
    Q_D(QwwHueSatRadialPicker);
    QPoint pt = me->pos();
    int r = d->radius(pt);
    if (r>=width()/2) return;
    d->m_pt = pt;
    int h = d->hue(pt);
    int s = (int)(r*255.0/(width()/2));
    emit colorPicked(QColor::fromHsv(h, s, d->m_value));
    update();
}

void QwwHueSatRadialPicker::mouseMoveEvent(QMouseEvent * me) {
    Q_D(QwwHueSatRadialPicker);
    QPoint pt = me->pos();
    int r = d->radius(pt);
    if (r>=width()/2) {
        r = width()/2;
        pt = d->m_pt;
    }
    d->m_pt = pt;
    int h = d->hue(pt);
    int s = (int)(r*255.0/(width()/2));
    emit colorPicked(QColor::fromHsv(h, s, d->m_value));
    update();
}

int QwwHueSatRadialPickerPrivate::radius(const QPoint & pt) const {
    Q_Q(const QwwHueSatRadialPicker);
    QPoint c = q->rect().center();
    int x = pt.x() - c.x();
    int y = pt.y() - c.y();
    int r = qRound(sqrt((double)(x*x + y*y)));

    return r;
}

/**
 *
 * @param p
 * @param pt
 */
void QwwHueSatRadialPicker::drawCrosshair(QPainter * p, const QPoint & pt) {
    if (pt.isNull()) return;
    p->save();
    p->setPen(Qt::black);
    p->drawLine(pt-QPoint(0, -3), pt-QPoint(0, -1));
    p->drawLine(pt-QPoint(0, 1), pt-QPoint(0, 3));
    p->drawLine(pt-QPoint(-3, 0), pt-QPoint(-1, 0));
    p->drawLine(pt-QPoint(1, 0), pt-QPoint(3, 0));
    p->restore();
}

int QwwHueSatRadialPickerPrivate::hue(const QPoint & pt) const {
    Q_Q(const QwwHueSatRadialPicker);
    QPoint c = q->rect().center();
    int x = c.x()-pt.x();
    int y = pt.y()-c.y();
    double a = qAbs(y)*1.0 / qAbs(x);   // tangent
    double rd = atan(a);
    double deg = rd * 180 / M_PI;
    int h = qRound(deg);
    if (x>=0 && y>=0) h=180+h;
    else if (x<0 && y>=0) h=360-h;
    else if (x<0 && y<0) h=h;
    else h=180-h;
    return h % 360;
}

/**
 *
 * @param c
 */
void QwwHueSatRadialPicker::setColor(const QColor & c) {
    Q_D(QwwHueSatRadialPicker);
    if (c==d->m_color) return;
    int h, s, v;
    c.getHsv(&h, &s, &v);
    if (v!=d->m_value) setValue(v);
    int r = qRound(s*1.0*(width()/2-2)/255.0);
    int x = qRound(r*cos(h*M_PI/180.0));
    int y = qRound(r*sin(h*M_PI/180.0));
    QPoint ctr = rect().center();
    d->m_pt = QPoint(x+ctr.x(), -y+ctr.y());
    d->m_color = c;
    emit colorPicked(c);
    update();
}

/**
 *
 * @return
 */
const QColor QwwHueSatRadialPicker::color() const {
    Q_D(const QwwHueSatRadialPicker);
    return d->m_color;
}

#endif
