//
// C++ Implementation: %{MODULE}
//
// Description:
//
//
// Author: %{AUTHOR} <%{EMAIL}>, (C) %{YEAR}
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "qwwnavigationbar.h"
#include <QPushButton>
#include <QButtonGroup>
#include <QStackedWidget>
#include <QLayout>
#include <QSplitter>
#include <QToolButton>
#include <QVariant>
#include <QMessageBox>
#include <QActionEvent>
#include "wwglobal_p.h"


/**
 *  @class  QwwNavigationBar
 *  @brief  Widget simmilar to MS Outlook navigation bar. (unstable)
 *
 */


class QwwNavigationBarPrivate : public QwwPrivate {
public:
  QwwNavigationBarPrivate(QwwNavigationBar *q) : QwwPrivate(q){
    stack = 0;
    buttonLayout = 0;
    bottomFrame = 0;
    topButton = 0;
    splitter = 0;
  }
  QList<QPushButton *> buttons;
  QStackedWidget *stack;
  QVBoxLayout *buttonLayout;
  QFrame *bottomFrame;
  QPushButton *topButton;
  QSplitter *splitter;
  void setTitle(QWidget *w, const QString &s);
  void setIcon(QWidget *w, const QIcon &ic);
};

QwwNavigationBar::QwwNavigationBar(QWidget *parent)
        : QWidget(parent), QwwPrivatable(new QwwNavigationBarPrivate(this)) {
    Q_D(QwwNavigationBar);
    d->stack = new QStackedWidget(this);
    while (d->stack->count()>0)
    d->stack->removeWidget(d->stack->widget(0));
    QWidget *bw = new QWidget;
    bw->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Preferred);
    d->buttonLayout = new QVBoxLayout(bw);
    d->buttonLayout->setSpacing(0);
    d->buttonLayout->setMargin(0);
    d->splitter = new QSplitter(Qt::Vertical);
//     splitter->setObjectName("__qt__passive_splitter");
    d->splitter->addWidget(d->stack);
    d->splitter->addWidget(bw);
    QVBoxLayout *l = new QVBoxLayout(this);
    d->topButton = new QPushButton;
    l->addWidget(d->topButton);
    d->topButton->setFlat(true);
    l->addWidget(d->splitter);
    l->setSpacing(0);
    l->setMargin(2);
    d->bottomFrame = new QFrame;
    d->bottomFrame->setFrameShape(QFrame::NoFrame);
    d->bottomFrame->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    l->addWidget(d->bottomFrame);
    QHBoxLayout *hl = new QHBoxLayout(d->bottomFrame);
    hl->setSpacing(2);
    hl->setMargin(1);
    hl->addStretch(1);
    connect(d->stack, SIGNAL(currentChanged(int)), this, SIGNAL(currentIndexChanged(int)));
}


QwwNavigationBar::~QwwNavigationBar() {}


/**
 * @brief Adds a widget to the navigation bar
 * @param child
 * @param icon
 * @param label
 */
void QwwNavigationBar::addWidget(QWidget * child, const QIcon &icon, const QString &label) {
    Q_D(QwwNavigationBar);
    insertWidget(d->stack->count(), child, icon, label);
}

/**
 *
 * @param child
 * @param label
 * @overload
 */
void QwwNavigationBar::addWidget(QWidget * child, const QString & label) {
    addWidget(child, QIcon(), label);
}



/**
 * @brief Inserts a widget to the navigation bar on a specified position
 * @param index
 * @param child
 * @param icon
 * @param label
 */
void QwwNavigationBar::insertWidget(int index, QWidget * child, const QIcon &icon, const QString &label) {
    Q_D(QwwNavigationBar);
    QPushButton *b = new QPushButton;
    connect(b, SIGNAL(clicked()), SLOT(_q_buttonClicked()));
    d->stack->insertWidget(index, child);
    d->buttonLayout->insertWidget(index, b);
    d->buttons.insert(index, b);

    b->setObjectName("__qt__passive_button");
    b->setIcon(child->windowIcon());

    b->updateGeometry();
    if(label.isNull()){
        b->setText(child->windowTitle());
    } else {
        d->setTitle(child, label);
    }
    int h = d->buttonLayout->parentWidget()->sizeHint().height();
    d->buttonLayout->parentWidget()->setMaximumSize(16777215, h);
    if (d->stack->count()==1) {
        setCurrentIndex(0);
    }
}

/**
 *
 * @param index
 * @param child
 * @param label
 * @overload
 */
void QwwNavigationBar::insertWidget(int index, QWidget * child, const QString & label) {
    insertWidget(index, child, QIcon(), label);
}

void QwwNavigationBar::_q_buttonClicked() {
    Q_D(QwwNavigationBar);
    QPushButton *b = static_cast<QPushButton*>(sender());
    int ind = d->buttons.indexOf(b);
    d->topButton->setText(b->text());
    d->topButton->setIcon(b->icon());
    d->stack->setCurrentIndex(ind);
}

/**
 * @brief Removes a widget from the navigation bar
 * @param index
 */
void QwwNavigationBar::removeWidget(int index) {
    Q_D(QwwNavigationBar);
    delete d->buttons.at(index);
    d->buttons.removeAt(index);
    d->stack->removeWidget(d->stack->widget(index));
    int ind = d->stack->currentIndex();
    if (ind>-1) {
        QPushButton *b = const_cast<QPushButton*>(button(ind));
        d->topButton->setText(b->text());
        d->topButton->setIcon(b->icon());
    } else {
        d->topButton->setText(QString::null);
        d->topButton->setIcon(QIcon());
    }
}

/**
 * @brief Returns an index of a widget in the navigation bar
 * @param w
 * @return
 */
int QwwNavigationBar::indexOf(QWidget *w) const {
    Q_D(const QwwNavigationBar);
    return d->stack->indexOf(w);
}

/**
 * @brief current index of the navigation bar
 * @return
 */
int QwwNavigationBar::currentIndex() const {
    Q_D(const QwwNavigationBar);
    return d->stack->currentIndex();
}

/**
 * @brief sets current index
 * @param index
 */
void QwwNavigationBar::setCurrentIndex(int index) {
    Q_D(QwwNavigationBar);
    if (index>=d->stack->count() || index<0) return;
    d->stack->setCurrentIndex(index);
    QWidget *w = widget(index);
    if(!w) return;
    QPushButton *b = const_cast<QPushButton*>(button(index));
    b->setText(w->windowTitle());
    d->topButton->setText(w->windowTitle());
    d->topButton->setIcon(w->windowIcon());
}

/**
 *
 * @param index
 * @return
 */
const QPushButton * QwwNavigationBar::button(int index) const {
    Q_D(const QwwNavigationBar);
    if (index>=d->buttons.count() || index<0) return 0;
    return d->buttons.at(index);
}

/**
 *
 * @param index
 * @return
 */
QIcon QwwNavigationBar::widgetIcon(int index) const {
    const QPushButton *b = button(index);
    if (!b) return QIcon();
    return b->icon();
}

/**
 *
 * @param index
 * @param icon
 */
void QwwNavigationBar::setWidgetIcon(int index, const QIcon &icon) {
    Q_D(QwwNavigationBar);
    QPushButton *b = const_cast<QPushButton*>(button(index));
    if (!b)
        return;
    b->setIcon(icon);
    QWidget *w = widget(index);
    if(!w) return;
    d->setIcon(w, icon);
    if (index == currentIndex()) {
        d->topButton->setIcon(icon);
    }
}

/**
 *
 * @param index
 * @return
 */
QString QwwNavigationBar::widgetLabel(int index) const {
    QWidget *w = widget(index);
    if(!w) return QString();
    return w->windowTitle();
}

/**
 *
 * @param index
 * @param label
 */
void QwwNavigationBar::setWidgetLabel(int index, const QString &label) {
    Q_D(QwwNavigationBar);
    QPushButton *b = const_cast<QPushButton*>(button(index));
    if (!b)
        return;
    b->setText(label);
    QWidget *w = widget(index);
    //if(!w) return;
    w->setProperty("windowTitle", QVariant(label));
    if (index == currentIndex()) {
        d->topButton->setText(label);
        emit widgetLabelChanged(label);
    }
}

/**
 *
 * @return
 */
QIcon QwwNavigationBar::currentWidgetIcon() const {
    const QPushButton *b = button(currentIndex());
    if (!b) return QIcon();
    return b->icon();
}

/**
 *
 * @param label
 */
void QwwNavigationBar::setCurrentWidgetLabel(const QString &label) {
    Q_D(QwwNavigationBar);
     if(label==currentWidgetLabel()) return;
    d->stack->currentWidget()->setWindowTitle(label);
    d->topButton->setText(label);
    QPushButton *b = const_cast<QPushButton*>(button(currentIndex()));
    b->setText(label);
    emit widgetLabelChanged(label);
}

/**
 *
 * @param icon
 */
void QwwNavigationBar::setCurrentWidgetIcon(const QIcon &icon) {
    Q_D(QwwNavigationBar);
//    if(icon==currentWidgetIcon()) return;
    d->stack->currentWidget()->setWindowIcon(icon);
    d->topButton->setIcon(icon);
    QPushButton *b = const_cast<QPushButton*>(button(currentIndex()));
    b->setIcon(icon);
    emit widgetIconChanged(icon);
}

/**
 *
 * @return
 */
QString QwwNavigationBar::currentWidgetLabel() const {
    return widgetLabel(currentIndex());
}

/**
 * @brief number of widgets in the navigation bar
 * @return
 */
int QwwNavigationBar::widgetCount() const {
    Q_D(const QwwNavigationBar);
    return d->stack->count();
}

/**
 *
 * @param index
 * @return
 */
QWidget * QwwNavigationBar::widget(int index) const {
    Q_D(const QwwNavigationBar);
    return d->stack->widget(index);
}

void QwwNavigationBarPrivate::setTitle(QWidget * w, const QString & s) {
    w->setWindowTitle(s);
}

void QwwNavigationBarPrivate::setIcon(QWidget * w, const QIcon & ic) {
    w->setWindowIcon(ic);
}

void QwwNavigationBar::actionEvent(QActionEvent * ev)
{
    Q_D(QwwNavigationBar);
    switch(ev->type()){
    case QEvent::ActionAdded: {


        QHBoxLayout *hl = static_cast<QHBoxLayout*>(d->bottomFrame->layout());
        if(ev->before()==0){
            QToolButton *tb = new QToolButton;
            tb->setDefaultAction(ev->action());
            tb->setAutoRaise(true);
            hl->addWidget(tb);
        } else
        for(int i=0;i<hl->count();i++){
            QLayoutItem *lit = hl->itemAt(i);
            if(!lit) break;
            QWidget *w = lit->widget();
            if(!w) continue;
            QToolButton *t = dynamic_cast<QToolButton*>(w);
            if(!t) continue;
            if(ev->before()==t->defaultAction()){
                QToolButton *tb = new QToolButton;
                tb->setDefaultAction(ev->action());
                tb->setAutoRaise(true);
                hl->insertWidget(i, tb);
                return;
            }
        }
    }
    break;
    case QEvent::ActionRemoved: {
        QHBoxLayout *hl = static_cast<QHBoxLayout*>(d->bottomFrame->layout());
        for(int i=0;i<hl->count();i++){
            QLayoutItem *lit = hl->itemAt(i);
            if(!lit) break;
            QWidget *w = lit->widget();
            if(!w) continue;
            QToolButton *t = dynamic_cast<QToolButton*>(w);
            if(!t) continue;
            if(t->defaultAction()==ev->action()){
                delete t;
                return;
            }
        }
    }
    break;
    default:break;
    }

}

