//
// C++ Implementation: %{MODULE}
//
// Description:
//
//
// Author: %{AUTHOR} <%{EMAIL}>, (C) %{YEAR}
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "qwwnumpad.h"
#include <QLayout>
#include <QButtonGroup>
#include <QToolButton>

/**
 *  @class  QwwNumPad
 *  @brief  A numeric keypad widget
 *
 */
/**
 * @fn      QwwNumPad::numberClicked(int);
 * @brief   numeric key clicked
 *
 * @fn      QwwNumPad::keyClicked(QString);
 * @brief   text of the key clicked
 *
 * @fn      QwwNumPad::hashClicked();
 * @brief   '#' key clicked
 *
 * @fn      QwwNumPad::asteriskClicked();
 * @brief   '*' key clicked
 *
 */

QwwNumPad::QwwNumPad(QWidget *parent) : QWidget(parent) {
    QGridLayout *l = new QGridLayout(this);
    l->setSpacing(2);
    l->setMargin(2);
    QButtonGroup *group = new QButtonGroup(this);
    for (int i=0;i<9;i++) {
        QToolButton *b = new QToolButton;
        b->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        b->setObjectName(QString("b%1").arg(i+1));
        b->setText(QString::number(i+1));
        l->addWidget(b, i/3, i%3);
        group->addButton(b, i+1);
    }
    QToolButton *b = new QToolButton;
    b->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
    b->setObjectName("b0");
    b->setText("0");
    l->addWidget(b, 3, 1);
    group->addButton(b, 0);

    b = new QToolButton;
    b->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
    b->setObjectName("bAster");
    b->setText("*");
    l->addWidget(b, 3, 0);
    group->addButton(b, 10);

    b = new QToolButton;
    b->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
    b->setText("#");
    b->setObjectName("bHash");
    l->addWidget(b, 3, 2);
    group->addButton(b, 11);
    connect(group, SIGNAL(buttonClicked(int)), SLOT(_q_clicked(int)));
    setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
}


QwwNumPad::~QwwNumPad() {}

void QwwNumPad::_q_clicked(int id) {
    if (id>=0 && id<10) {
        emit numberClicked(id);
        emit keyClicked(QString::number(id));
        return;
    }
    if (id==10) {
        emit asteriskClicked();
        emit keyClicked("*");
        return;
    }
    if (id==11) {
        emit hashClicked();
        emit keyClicked("#");
        return;
    }
}

/**
 * @brief Performs an animated click.
 * @param button button text to click
 * @param msec button time to keep the key pressed
 *
 * The \a button is pressed immediately, and released \a msec milliseconds later
 * (the default is 100 ms).
 *
 * Calling this function again on the same button before it was released will reset
 * the release timer.
 * All signals associated with a click are emitted as appropriate.
 * This function does nothing if the button is disabled.
 */
void QwwNumPad::animateClick(const QString & b, int msec) {
    if (b.size()!=1) return;
    char key = b[0].toAscii();
    QList<QToolButton*> items;
    QString name;
    switch (key) {
    case '#':
        name = "bHash";
        break;
    case '*':
        name = "bAster";
        break;
    default:
        name = QString("b%1").arg(key);
    }
    QToolButton *button = qFindChild<QToolButton*>(this, name);
    if (!button) return;
    button->animateClick(msec);
}


