//
// C++ Implementation: %{MODULE}
//
// Description:
//
//
// Author: %{AUTHOR} <%{EMAIL}>, (C) %{YEAR}
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef WW_NO_TASKPANEL
#include "qwwtaskpanel.h"
#include "qwwtaskpanel_p.h"

TaskHeader::TaskHeader(QWidget *w, QWidget *parent) : QFrame(parent) {
    m_widget = w;
    setFrameShape(QFrame::StyledPanel);
    setFrameRect(rect().adjusted(0, 8, 0, 0));
    setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    QHBoxLayout *l = new QHBoxLayout(this);
    l->setMargin(1);
    m_spacer = new QSpacerItem(0, 0, QSizePolicy::Fixed, QSizePolicy::Fixed);
    l->addItem(m_spacer);
    m_text = new QLabel;
    QFont f = m_text->font();
    f.setBold(true);
    m_text->setFont(f);
    m_button = new QToolButton;
    m_button->setObjectName("__qt__passive_button");
    m_button->setAutoRaise(true);
    m_button->setCheckable(true);
    m_button->setArrowType(Qt::DownArrow);
    l->addWidget(m_text);
    l->addWidget(m_button);
}

void TaskHeader::setToggleIcon(const QIcon &i) {
    m_button->setIcon(i);
    if (i.isNull()) {
        m_button->setArrowType(m_button->isChecked() ? Qt::UpArrow : Qt::DownArrow);
    } else {
        m_button->setArrowType(Qt::NoArrow);
    }
}

void TaskHeader::setTaskName(const QString &n) {
    m_text->setText(n);
    layout()->invalidate();
    update();
}

void TaskHeader::setIcon(const QIcon &i) {
    m_icon = i;
    if (i.isNull()) {
        m_spacer->changeSize(0,0, QSizePolicy::Fixed, QSizePolicy::Fixed);
    } else {
        m_spacer->changeSize(50,16, QSizePolicy::Fixed, QSizePolicy::Fixed);
    }
    m_spacer->invalidate();
    layout()->invalidate();
    update();
}

void TaskHeader::paintEvent(QPaintEvent *ev) {
    QFrame::paintEvent(ev);
    QPainter p(this);
    m_icon.paint(&p, QRect(2, 1, 32, 32), Qt::AlignCenter, isEnabled() ? QIcon::Normal : QIcon::Disabled);
}

QToolButton* TaskHeader::toggleButton() const {
    return m_button;
}




Task::Task(QWidget *body, QWidget *parent) : QWidget(parent) {
    m_body = body;
    m_animBody = 0;
    m_animator.setDuration(3000);
    m_animator.setCurveShape(QTimeLine::EaseInOutCurve);
    QVBoxLayout *l = new QVBoxLayout(this);
    l->setSpacing(0);
    l->setMargin(0);
    m_header = new TaskHeader(body);

    l->addWidget(m_header);
    l->addWidget(m_body);

    m_body->setVisible(false);
    m_body->installEventFilter(this);
    connect(m_header->toggleButton(), SIGNAL(toggled(bool)), this, SLOT(setOpen(bool)));
    connect(&m_animator, SIGNAL(frameChanged(int)), SLOT(animate(int)));
    connect(&m_animator, SIGNAL(finished()), SLOT(animFinished()));
}

void Task::setOpen(bool o) {
    QToolButton *b = m_header->toggleButton();
    if (b->isChecked() == o) {
        b->setChecked(o);
        if (b->arrowType()!=Qt::NoArrow) {
            if (o) {
                b->setArrowType(Qt::UpArrow);
            } else {
                b->setArrowType(Qt::DownArrow);
            }
        }
        if (m_animator.state()!=QTimeLine::NotRunning) {
            m_animator.setDirection(m_animator.direction()==QTimeLine::Forward ? QTimeLine::Backward : QTimeLine::Forward);
        } else {
            m_animBody = new QWidget;
            m_animBody->installEventFilter(this);
#ifndef Q_WS_WIN
            m_animBody->setEnabled(false);
#endif
            m_animBody->setAttribute(Qt::WA_NoSystemBackground, true);
            m_body->show();
            m_animpix = QPixmap::grabWidget(this);

            if (o) {
                m_animator.setDirection(QTimeLine::Forward);
                m_animBody->setFixedSize(width(), 0);
                m_animator.setFrameRange(0, m_body->sizeHint().height());
            } else {
                m_animator.setDirection(QTimeLine::Backward);
                m_animBody->setFixedSize(width(), m_body->width());
                m_animator.setFrameRange(0, m_body->height());
            }
            m_body->hide();
            QVBoxLayout *l = (QVBoxLayout*)layout();
            l->addWidget(m_animBody);
            m_animBody->show();
            m_animator.start();
        }
    }
}

bool Task::eventFilter(QObject *o, QEvent *e) {
    if (o==m_animBody && e->type()==QEvent::Paint) {
        QPainter p(m_animBody);
        p.drawPixmap(m_animBody->rect(), m_animpix, m_animpix.rect().adjusted(0, m_animpix.height()-m_animBody->height(), 0, 0));
        return true;
    }
//     if (o==m_body && e->type()==QEvent::WindowTitleChange) {
//         setName(m_body->windowTitle());
//     }
//     if (o==m_body && e->type()==QEvent::WindowIconChange) {
//         setIcon(m_body->windowIcon());
//     }
    return false;
}

void Task::animate(int frame ) {
    m_animBody->setFixedSize(width(), frame);
    m_animBody->updateGeometry();
    m_animBody->update();
//     qDebug("Ramka: %d", frame);
}

void Task::animFinished() {
    m_animBody->hide();
    delete m_animBody;
    m_animBody = 0;
    if (m_animator.currentFrame()!=0) {
        m_body->show();
    }
}


/**
 *  @class QwwTaskPanel
 *  @brief Task panel simmilar to the one from WindowsXP.(unstable)
 *
 *
 *
 */

QwwTaskPanel::QwwTaskPanel(QWidget *parent)
        : QScrollArea(parent) {
//      m_animated = true;
    m_panel = new QWidget(viewport());
    m_panel->setObjectName("ww_taskpanel_panel");
    m_current = -1;
    QVBoxLayout *l = new QVBoxLayout(m_panel);

    l->addStretch(0);
    setWidget(m_panel);
    setWidgetResizable(true);
    setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Expanding);
}


QwwTaskPanel::~QwwTaskPanel() {}

void QwwTaskPanel::addTask(QWidget * task, const QString & label) {
    insertTask(taskCount(), task, label);
}

void QwwTaskPanel::addTask(QWidget * task, const QIcon & icon, const QString & label) {
    insertTask(taskCount(), task, icon, label);
}

void QwwTaskPanel::insertTask(int index, QWidget * task, const QString & label) {
    insertTask(index, task, QIcon(), label);
}

void QwwTaskPanel::insertTask(int index, QWidget * task, const QIcon & icon, const QString & label) {
    if (!task) return;
    Task *tsk = new Task(task);
    tsk->setToggleIcon(m_toggleIcon);
    if (label.isNull())
        tsk->setName(task->windowTitle());
    else {
        tsk->setName(label);
        task->setWindowTitle(label);
    }
    if (icon.isNull()) {
        tsk->setIcon(task->windowIcon());
    } else {
        tsk->setIcon(icon);
        task->setWindowIcon(icon);
    }
    static_cast<QBoxLayout*>(m_panel->layout())->insertWidget(index, tsk);
    m_tasks.insert(index, tsk);

    if (m_tasks.count()==1) {
        setCurrentIndex(0);
    }
     tsk->show();
}

int QwwTaskPanel::taskCount() const {
    return m_tasks.count();
}

void QwwTaskPanel::removeTask(int index) {
    if (index < 0 || index>=m_tasks.count()) return;
    Task *tsk = static_cast<Task*>(m_tasks.at(index));
    m_tasks.removeAt(index);
    if (m_tasks.count()<=index) {
        setCurrentIndex(m_tasks.count()-1);
    }
    QWidget *body = tsk->body();
    body->setParent(this);
    delete tsk;
}

QWidget * QwwTaskPanel::task(int index) const {
    if (index < 0 || index>=m_tasks.count()) return 0;
    Task *tsk = static_cast<Task*>(m_tasks.at(index));
    return tsk ? tsk->body() : 0;
}

int QwwTaskPanel::indexOf(QWidget * task) const {
    for (int i=0;i<m_tasks.count();i++) {
        Task *tsk = static_cast<Task*>(m_tasks.at(i));
        if (task == tsk) return i;
    }
    return -1;
}

void QwwTaskPanel::setToggleIcon(const QIcon & icon) {
    m_toggleIcon = icon;
    foreach(QWidget *tsk, m_tasks) {
        static_cast<Task*>(tsk)->setToggleIcon(icon);
    }
}

void QwwTaskPanel::setCurrentIndex(int index) {
    if (index<0 || index>=m_tasks.count() || index==m_current)
        return;
    if (m_current!=-1) {
        Task *tsk = static_cast<Task*>(m_tasks.at(m_current));
        tsk->setOpen(false);
    }
    m_current = index;
    Task *tsk = static_cast<Task*>(m_tasks.at(index));
    tsk->setOpen(true);
    emit currentIndexChanged(index);
}

QWidget * QwwTaskPanel::currentTask() const {
    return task(currentIndex());
}

void QwwTaskPanel::setTaskIcon(int index, const QIcon & icon) {
    if (index < 0 || index>=m_tasks.count()) return;
    Task *tsk = dynamic_cast<Task*>(m_tasks.at(index));
    if (!tsk) return;
    tsk->setIcon(icon);

}

void QwwTaskPanel::setTaskTitle(int index, const QString & title) {
    if (index < 0 || index>=m_tasks.count()) return;
    Task *tsk = dynamic_cast<Task*>(m_tasks.at(index));
    if (!tsk) return;
    tsk->setName(title);
}

void QwwTaskPanel::setTaskName(int index, const QString & name) {
    if (index < 0 || index>=m_tasks.count()) return;
    Task *tsk = dynamic_cast<Task*>(m_tasks.at(index));
    if (!tsk) return;
    tsk->body()->setObjectName(name);
}
#endif
