//
// C++ Implementation: %{MODULE}
//
// Description:
//
//
// Author: %{AUTHOR} <%{EMAIL}>, (C) %{YEAR}
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef WW_NO_TWOCOLORINDICATOR

#include "qwwtwocolorindicator.h"
#include <QStylePainter>
#include <QStyleOptionFrame>
#include <QMouseEvent>
#include <QColorDialog>
#include "wwglobal_p.h"

/**
 * @class QwwTwoColorIndicator
 * @brief A widget allowing to choose a foreground and background colour.
 *
 *
 *
 */
/**
 * @property QColor QwwTwoColorIndicator::fgColor
 * @brief foreground colour
 * @sa setFgColor, fgColor
 */
 /**
 * @property QColor QwwTwoColorIndicator::bgColor
 * @brief background colour
 * @sa setBgColor, bgColor
 */
 /**
 * @property bool QwwTwoColorIndicator::active
 * @brief whether mouse click enables colour select
 */
 /**
 *  @fn QwwTwoColorIndicator::fgChanged(const QColor &)
 *  @brief foreground colour changed
 *
 *  @fn QwwTwoColorIndicator::bgChanged(const QColor &)
 *  @brief background colour changed
 *
 *  @fn QwwTwoColorIndicator::fgClicked()
 *  @brief foreground button was clicked
 *
 *  @fn QwwTwoColorIndicator::bgClicked()
 *  @brief background button was clicked
 *
 *  @fn QwwTwoColorIndicator::fgPressed()
 *  @brief foreground button was pressed
 *
 *  @fn QwwTwoColorIndicator::bgPressed()
 *  @brief background button was pressed
 *
 *
 */

/**
 * \internal
 */
class QwwTwoColorIndicatorPrivate : public QwwPrivate{
public:
    QwwTwoColorIndicatorPrivate(QwwPrivatable *p) : QwwPrivate(p),
    fg(Qt::black), bg(Qt::white){
        fgP = false;
        bgP = false;
        active = true;
    }
    QColor fg;
    QColor bg;
    bool fgP;
    bool bgP;
    bool active;
};

/**
 *
 * @param parent
 */
QwwTwoColorIndicator::QwwTwoColorIndicator(QWidget *parent)
        : QWidget(parent), QwwPrivatable(new QwwTwoColorIndicatorPrivate(this)){
    setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
}


/**
 *
 */
QwwTwoColorIndicator::~QwwTwoColorIndicator() {
}

/**
 * @internal
 * @brief paint event
 * @param
 */
void QwwTwoColorIndicator::paintEvent(QPaintEvent *) {
    Q_D(QwwTwoColorIndicator);
    QPainter painter(this);
    paintSection(&painter, QRect(width()/3, height()/3, 2*width()/3, 2*height()/3), d->bg);
    paintSection(&painter, QRect(0, 0, 2*width()/3, 2*height()/3), d->fg);
}


/**
 *
 * @param painter
 * @param rect
 * @param color
 */
void QwwTwoColorIndicator::paintSection(QPainter * painter, const QRect & rect, const QColor & color) {
    QStyleOptionFrame opt;
    opt.initFrom(this);
    opt.rect = rect;
    painter->fillRect(opt.rect.adjusted(1, 1, -1, -1), opt.state & QStyle::State_Enabled ? color : palette().color(QPalette::Disabled, QPalette::Window ));
    style()->drawPrimitive(QStyle::PE_Frame, &opt, painter, this);
}


/**
 *  @brief Swaps foreground and background colours.
 */
void QwwTwoColorIndicator::switchColors() {
    Q_D(QwwTwoColorIndicator);
    if (d->fg==d->bg) return;
    qSwap(d->fg, d->bg);
    emit bgChanged(d->bg);
    emit fgChanged(d->fg);
    update();
}

/**
 *  @brief Sets the background colour
 *  @param b colour to set
 */
void QwwTwoColorIndicator::setBgColor(const QColor &b) {
    Q_D(QwwTwoColorIndicator);
    if (b==d->bg) return;
    d->bg = b;
    emit bgChanged(b);
    update();
}

/**
 *  @brief  Sets the foreground colour
 *  @param f colour to set
 */
void QwwTwoColorIndicator::setFgColor(const QColor &f) {
    Q_D(QwwTwoColorIndicator);
    if (f==d->fg) return;
    d->fg = f;
    emit fgChanged(f);
    update();
}



const QColor & QwwTwoColorIndicator::bgColor() const {
    Q_D(const QwwTwoColorIndicator);
    return d->bg;
}

const QColor & QwwTwoColorIndicator::fgColor() const {
    Q_D(const QwwTwoColorIndicator);
    return d->fg;
}

/**
 *
 *  @brief
 *  @return
 */
bool QwwTwoColorIndicator::isActive() const {
    Q_D(const QwwTwoColorIndicator);
    return d->active;
}

/**
 *
 *
 *  @sa isActive, active
 */
void QwwTwoColorIndicator::setActive(bool a) {
    Q_D(QwwTwoColorIndicator);
    d->active = a;
}


/**
 * @internal
 */
QSize QwwTwoColorIndicator::sizeHint() const {
    return QSize(40,40);
}


/**
 * @internal
 */
QSize QwwTwoColorIndicator::minimumSizeHint() const {
    return QSize(20,20);
}

/**
 * @internal
 */
void QwwTwoColorIndicator::mousePressEvent(QMouseEvent *ev) {
    Q_D(QwwTwoColorIndicator);
    int w = width()/3;
    int h = height()/3;
    if (ev->button() == Qt::LeftButton) {
        if (QRect(0, 0, 2*w, 2*h).contains(ev->pos())) {
            d->fgP = true;
            d->bgP = false;
            emit fgPressed();
        } else if (QRect(w, h, 2*w, 2*h).contains(ev->pos())) {
            d->bgP = true;
            d->fgP = false;
            emit bgPressed();
        }
    }
}

/**
 * @internal
 */
void QwwTwoColorIndicator::mouseReleaseEvent(QMouseEvent *ev) {
    Q_D(QwwTwoColorIndicator);
    int w = width()/3;
    int h = height()/3;
    if (ev->button() == Qt::LeftButton) {
        if (d->fgP && QRect(0, 0, 2*w, 2*h).contains(ev->pos())) {
            emit fgClicked();
            if (d->active) {
                QColor c = QColorDialog::getColor(d->fg, this);
                if (c.isValid()) {
                    setFgColor(c);
                }
            }
        } else if (d->bgP && QRect(w, h, 2*w, 2*h).contains(ev->pos())) {
            emit bgClicked();
            if (d->active) {
                QColor c = QColorDialog::getColor(d->bg, this);
                if (c.isValid()) {
                    setBgColor(c);
                }
            }
        }
        d->fgP = d->bgP = false;
    }
}

#endif
